<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace SlackModule\Controller;

use App\Lib\Exceptions\MissingDbBackendException;
use App\Model\Table\ProxiesTable;
use Cake\Http\Exception\MethodNotAllowedException;
use Cake\ORM\TableRegistry;
use DateTime;
use GuzzleHttp\Client;
use itnovum\openITCOCKPIT\Core\DbBackend;
use itnovum\openITCOCKPIT\Core\HoststatusFields;
use itnovum\openITCOCKPIT\Core\ServicestatusFields;
use itnovum\openITCOCKPIT\Core\System\Gearman;
use itnovum\openITCOCKPIT\Exceptions\HostNotFoundException;
use itnovum\openITCOCKPIT\Exceptions\ServiceNotFoundException;
use SlackModule\Lib\SlackData;
use SlackModule\Model\Table\SlackSettingsTable;

class AcknowledgeController extends AppController {

    private $proxySettings;
    private $slackSettings;

    /**
     * @throws MissingDbBackendException
     * @throws HostNotFoundException
     * @throws ServiceNotFoundException
     */
    public function submit() {
        $this->DbBackend = new DbBackend();
        /** @var SlackSettingsTable $SlackSettingsTable */
        $SlackSettingsTable = TableRegistry::getTableLocator()->get('SlackModule.SlackSettings');
        /** @var ProxiesTable $ProxiesTable */
        $ProxiesTable = TableRegistry::getTableLocator()->get('Proxies');
        /** @var SlackData $SlackData */
        $SlackData = new SlackData();
        /** @var Gearman $GearmanClient */
        $GearmanClient = new Gearman();
        /** @var Client $guzzleclient */
        $guzzleclient = new Client();

        $HoststatusTable = $this->DbBackend->getHoststatusTable();
        $ServicestatusTable = $this->DbBackend->getServicestatusTable();

        $this->proxySettings = $ProxiesTable->getSettings();
        $this->slackSettings = $SlackSettingsTable->getSlackSettings();

        if (!$this->request->is('post') || $this->slackSettings['two_way'] != 1) {
            throw new MethodNotAllowedException();
        }

        $this->autoRender = false;
        $getArray = json_decode($this->request->getData('payload'), true);
        $actionsData = $getArray['actions'][0];
        $userId = $getArray['user']['id'];
        $OAuthAccessToken = $this->slackSettings['oauth_access_token'];
        $userInfoUrl = sprintf('https://slack.com/api/users.info?token=%s&user=%s&pretty=1', $OAuthAccessToken, $userId);


        $options['headers'] = [
            'Accept' => 'application/json'
        ];
        if ($this->slackSettings['use_proxy']) {
            $options['proxy'] = [
                'http'  => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port']),
                'https' => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port'])
            ];
        } else {
            $options['proxy'] = [
                'http'  => false,
                'https' => false
            ];
        }

        $response = $guzzleclient->post($userInfoUrl, $options);
        $userJson = $response->getBody();


        $userRealName = json_decode($userJson)->user->real_name;
        $date = new DateTime();
        $values = explode('||', $actionsData['value']);

        /*
            $values[0]="acknowledged"
            $values[1]=notification type
            $values[2]=host-uuid
            $values[3]=service-uuid if notification type == Service
        */

        if (strpos($getArray['original_message']['attachments'][1]['text'], '*Already acknowledged* by') !== false) {
            $acknowledgeText = $getArray['original_message']['attachments'][1]['text'] . ", " . $userRealName;
            $acknowledgeColor = $getArray['original_message']['attachments'][1]['color'];
        } else {
            $acknowledgeText = '*Already acknowledged* by ' . $userRealName;
            $acknowledgeColor = $getArray['original_message']['attachments'][1]['color'];
        }
        $output = $getArray['original_message']['attachments'][0]['text'];

        $acknowledgeTime = $date->getTimestamp();
        $acknowledgeButtonType = 'button';
        $HoststatusFields = new HoststatusFields($this->DbBackend);
        $HoststatusFields->currentState();
        if ($values[1] == "Host" && $HoststatusTable->byUuid($values[2], $HoststatusFields)['Hoststatus']['current_state'] == "0") {
            $acknowledgeTime = $getArray['action_ts'];
            $acknowledgeText = $getArray['original_message']['attachments'][1]['text'] . "\n\n*" . $values[1] . " is back!*";
            $acknowledgeButtonType = 'disabled';
            $acknowledgeColor = '#5cb85c';
        }
        $ServicestatusFields = new ServicestatusFields($this->DbBackend);
        $ServicestatusFields->currentState();
        if ($values[1] == "Service" && $ServicestatusTable->byUuid($values[3], $ServicestatusFields)['Servicestatus']['current_state'] == "0") {
            $acknowledgeTime = $getArray['action_ts'];
            $acknowledgeText = $getArray['original_message']['attachments'][1]['text'] . "\n\n*" . $values[1] . " is back!*";
            $acknowledgeButtonType = 'disabled';
            $acknowledgeColor = '#5cb85c';
        }

        $acknowledgeValues = $values[0] . '||' . $values[1] . '||' . $values[2];
        if ($values[1] == "Service") {
            $acknowledgeValues .= '||' . $values[3];
        }


        if ($values[1] == "Host") {
            $host = $SlackData->getHost($values[2]);

            $GearmanClient->sendBackground('cmd_external_command', [
                'command'     => 'ACKNOWLEDGE_HOST_PROBLEM',
                'parameters'  => [
                    'hostUuid'   => $host['uuid'],
                    'sticky'     => 2, // 2 = sticky with Nagios Core, 1 = Sticky with Naemon Core, 2 works for both :)
                    'notify'     => 0, // do not enable - loop!
                    'persistent' => 1,
                    'author'     => $userRealName,
                    'comment'    => sprintf(
                        'Issue was acknowledged by %s due to Slack.',
                        $userRealName
                    )
                ],
                'satelliteId' => null
            ]);

        } else if ($values[1] == "Service") {
            $service = $SlackData->getService($values[3]);

            $GearmanClient->sendBackground('cmd_external_command', [
                'command'     => 'ACKNOWLEDGE_SVC_PROBLEM',
                'parameters'  => [
                    'hostUuid'    => $values[2],
                    'serviceUuid' => $service['uuid'],
                    'sticky'      => 2, // 2 = sticky with Nagios Core, 1 = Sticky with Naemon Core, 2 works for both :)
                    'notify'      => 0, // do not enable - loop!
                    'persistent'  => 1,
                    'author'      => $userRealName,
                    'comment'     => sprintf(
                        'Issue was acknowledged by %s due to Slack.',
                        $userRealName
                    )
                ],
                'satelliteId' => null
            ]);
        }

        $newJsonArray = [
            'response_type'    => 'ephemeral',
            'replace_original' => true,
            'text'             => $getArray['original_message']['text'],
            'attachments'      => [
                [
                    'text'            => $output,
                    'fallback'        => 'No output',
                    'color'           => $acknowledgeColor,
                    'callback_id'     => 'host_output',
                    'attachment_type' => 'default',
                ],
                [
                    'text'            => $acknowledgeText,
                    "mrkdwn_in"       => ["text"],
                    'callback_id'     => 'ack_host',
                    'color'           => $acknowledgeColor,
                    'attachment_type' => 'default',
                    'actions'         => [
                        [
                            'name'  => 'ack_button',
                            'text'  => 'Acknowledge too',
                            'style' => 'primary',
                            'type'  => $acknowledgeButtonType,
                            'value' => $acknowledgeValues,
                        ]
                    ],
                    'footer'          => 'Last acknowledged',
                    'footer_icon'     => 'https://pbs.twimg.com/profile_images/692366534623256576/158ywUkg.jpg',
                    "ts"              => $acknowledgeTime,
                ]
            ],
        ];

        $this->setResponse($this->getResponse()->withStringBody(json_encode($newJsonArray, true))->withType('application/json'));
    }
}

