<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace SlackModule\Model\Table;

use Cake\Datasource\EntityInterface;
use Cake\ORM\Behavior\TimestampBehavior;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use SlackModule\Model\Entity\SlackSetting;

/**
 * SlackSettings Model
 *
 * @method SlackSetting get($primaryKey, $options = [])
 * @method SlackSetting newEntity($data = null, array $options = [])
 * @method SlackSetting[] newEntities(array $data, array $options = [])
 * @method SlackSetting|false save(EntityInterface $entity, $options = [])
 * @method SlackSetting saveOrFail(EntityInterface $entity, $options = [])
 * @method SlackSetting patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method SlackSetting[] patchEntities($entities, array $data, array $options = [])
 * @method SlackSetting findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin TimestampBehavior
 */
class SlackSettingsTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('slack_settings');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator {
        $validator
            ->nonNegativeInteger('id')
            ->allowEmptyString('id', null, 'create');

        $validator
            ->scalar('api_url')
            ->maxLength('api_url', 255)
            ->allowEmptyString('api_url');

        $validator
            ->scalar('oauth_access_token')
            ->maxLength('oauth_access_token', 255)
            ->allowEmptyString('oauth_access_token');

        $validator
            ->boolean('two_way')
            ->notEmptyString('two_way');

        $validator
            ->boolean('use_proxy')
            ->notEmptyString('use_proxy');

        return $validator;
    }

    /**
     * @param bool $toArray
     * @return array|EntityInterface|SlackSetting|null
     */
    public function getSlackSettings($toArray = false) {
        $default = [
            'api_url'            => null,
            'oauth_access_token' => null,
            'two_way'            => false,
            'use_proxy'          => false
        ];

        $result = $this->find()->first();

        if (empty($result)) {
            $result = $this->newEntity($default);
        }
        if ($toArray) {
            return $result->toArray();
        }

        return $result;
    }
}
